'use strict';

var generateScriptableBanner = require('@scriptables/manifest/generateScriptableBanner');
var validateManifest = require('@scriptables/manifest/validateManifest');
var fs = require('fs');
var promises = require('fs/promises');
var path = require('path');
var micromatch = require('micromatch');
var findicloud = require('findicloud');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var micromatch__default = /*#__PURE__*/_interopDefault(micromatch);

// src/scriptable-banner.ts

// src/logger.ts
var levels = {
  verbose: 0,
  info: 1,
  warn: 2,
  error: 3
};
var Logger = class {
  namespace;
  level;
  enabled;
  constructor(namespace, options = {}) {
    this.namespace = namespace;
    this.level = options.level || "info";
    this.enabled = options.enabled ?? true;
  }
  shouldLog(msgLevel) {
    return this.enabled && levels[msgLevel] >= levels[this.level];
  }
  verbose(...args) {
    if (this.shouldLog("verbose")) {
      console.log(`[${this.namespace}]`, ...args);
    }
  }
  info(...args) {
    if (this.shouldLog("info")) {
      console.log(`[${this.namespace}]`, ...args);
    }
  }
  warn(...args) {
    if (this.shouldLog("warn")) {
      console.warn(`[${this.namespace}]`, ...args);
    }
  }
  error(...args) {
    if (this.shouldLog("error")) {
      console.error(`[${this.namespace}]`, ...args);
    }
  }
};
function createLogger(namespace, options) {
  return new Logger(namespace, options);
}
function createPatternMatcher(patterns) {
  if (!patterns) return () => true;
  const patternArray = Array.isArray(patterns) ? patterns : [patterns];
  const includePatterns = patternArray.filter((p) => !p.startsWith("!"));
  const excludePatterns = patternArray.filter((p) => p.startsWith("!")).map((p) => p.slice(1));
  return (fileName) => {
    return micromatch__default.default.isMatch(fileName, includePatterns, {
      dot: true,
      ignore: excludePatterns
    });
  };
}
function isMatch(fileName, pattern, options) {
  return micromatch__default.default.isMatch(fileName, pattern, options);
}
async function waitForFile(filePath, timeout = 5e3) {
  const start = Date.now();
  while (Date.now() - start < timeout)
    if (fs.existsSync(filePath)) {
      try {
        await promises.readFile(filePath, "utf-8");
        return;
      } catch {
        await new Promise((resolve3) => setTimeout(resolve3, 100));
      }
    }
  throw new Error(`File not available after ${timeout}ms: ${filePath}`);
}
function normalizePath(path) {
  return path.replace(/\\/g, "/");
}
function isPathEqual(path1, path2) {
  return path.resolve(path1) === path.resolve(path2);
}

// src/scriptable-banner.ts
var DEFAULT_MANIFEST_EXTENSIONS = [".manifest.json", ".manifest", ".json"];
function scriptableBanner(options = {}) {
  const {
    manifestExtensions = DEFAULT_MANIFEST_EXTENSIONS,
    warnOnMissingManifest = true,
    warnOnInvalidManifest = true,
    warnOnMissingEntry = true,
    getManifestPath,
    patterns = ["**/*.{ts,tsx,js,jsx}"]
  } = options;
  const logger = createLogger("scriptable-banner");
  return {
    name: "scriptable",
    setup(build) {
      let manifestCache = /* @__PURE__ */ new Map();
      if (!build.initialOptions.metafile) {
        logger.warn(
          'Warning: "metafile" option was not enabled. Automatically setting "metafile: true" to allow scriptable plugin to modify output files.'
        );
        build.initialOptions.metafile = true;
      }
      build.onStart(() => {
        manifestCache.clear();
      });
      build.onResolve({ filter: /.*/ }, async (args) => {
        if (!isMatch(args.path, patterns)) {
          return null;
        }
        if (Array.isArray(build.initialOptions.entryPoints) && build.initialOptions.entryPoints.map(String).includes(args.path)) {
          const dir = path.dirname(args.path);
          const baseFileName = path.basename(args.path).replace(/\.[tj]sx?$/, "");
          const manifestPath = getManifestPath ? getManifestPath(args.path) : manifestExtensions.map((ext) => path.resolve(dir, `${baseFileName}${ext}`)).find(fs.existsSync);
          if (manifestPath) {
            try {
              const manifestContent = await promises.readFile(manifestPath, "utf8");
              const manifest = JSON.parse(manifestContent);
              const normalizedPath = normalizePath(args.path);
              try {
                validateManifest.validateManifest(manifest);
              } catch (error) {
                if (warnOnInvalidManifest && error instanceof Error) {
                  logger.warn(`Invalid manifest file: ${manifestPath} - ${error.message}`, error);
                }
                return null;
              }
              manifestCache.set(normalizedPath, manifest);
            } catch (error) {
              logger.warn(`Failed to read manifest file: ${manifestPath}`, error);
            }
          } else if (warnOnMissingManifest) {
            logger.warn(`No manifest file found for entry point: ${args.path}`);
          }
        }
        return null;
      });
      build.onEnd(async (result) => {
        if (!result.metafile || Object.keys(result.metafile.outputs).length === 0) {
          logger.warn(
            'Skipping banner addition because "metafile" is not available or contains no outputs. Ensure "metafile: true" is set in esbuild options if write is enabled.'
          );
          return;
        }
        const outputs = result.metafile.outputs;
        for (const outputPath of Object.keys(outputs)) {
          const outputMeta = outputs[outputPath];
          const entryPoint = outputMeta?.entryPoint ?? "";
          const normalizedEntryPath = entryPoint ? path.posix.normalize(entryPoint.split(path.sep).join("/")) : "";
          if (!normalizedEntryPath || !manifestCache.has(normalizedEntryPath)) {
            if (warnOnMissingEntry) {
              logger.warn(`No matching entry point found for output file: ${outputPath}`);
            }
            continue;
          }
          try {
            const manifest = manifestCache.get(normalizedEntryPath) || {};
            const banner = generateScriptableBanner.generateScriptableBanner(manifest);
            if (result.outputFiles) {
              const outputFile = result.outputFiles.find((file) => file.path === outputPath || file.path === "<stdout>");
              if (outputFile) {
                outputFile.contents = Buffer.from(banner + outputFile.text.trimEnd());
              }
            } else {
              try {
                await waitForFile(outputPath);
                const content = await promises.readFile(outputPath, "utf-8");
                const updatedContent = banner + content.trimEnd();
                await promises.writeFile(outputPath, updatedContent, "utf8");
              } catch (error) {
                logger.error(
                  `Failed to process file ${outputPath}: ${error instanceof Error ? error.message : String(error)}`
                );
              }
            }
          } catch (error) {
            logger.error(`Failed to process output file: ${outputPath}`, error);
          }
        }
      });
    }
  };
}
var findICloudBugsUrl = "https://github.com/hexxspark/findicloud/issues";
async function tryReadManifest(filePath, extensions) {
  for (const ext of extensions) {
    try {
      const manifestPath = filePath.replace(/\.[^.]+$/, "") + ext;
      const content = await promises.readFile(manifestPath, "utf-8");
      return JSON.parse(content);
    } catch {
      continue;
    }
  }
  return null;
}
function scriptableDeploy(options = {}) {
  const {
    verbose = false,
    continueOnError = true,
    scriptableDir,
    patterns = ["**/*.{ts,tsx,js,jsx}"],
    addBanner = true,
    manifestExtensions = [".manifest.json", ".manifest", ".json"]
  } = options;
  const filterFile = createPatternMatcher(patterns);
  const logger = createLogger("scriptable-deploy", {
    level: verbose ? "verbose" : "info"
  });
  let targetDir;
  return {
    name: "scriptable-deploy",
    setup(build) {
      if (!build.initialOptions.metafile) {
        logger.verbose("Automatically enabling metafile option");
        build.initialOptions.metafile = true;
      }
      build.onStart(async () => {
        try {
          if (scriptableDir) {
            targetDir = scriptableDir;
          } else {
            const [found] = await findicloud.findICloudDrivePaths({ appNamePattern: "Scriptable", types: [findicloud.PathType.APP_STORAGE] });
            targetDir = found?.path;
          }
          if (!targetDir) {
            throw new Error(
              `Scriptable directory not found in iCloud Drive. Please ensure iCloud Drive is installed and Scriptable app is synced with iCloud. If iCloud Drive is not installed, you can download it from: https://support.apple.com/en-us/HT204283. If the issue persists, please report it at: ${findICloudBugsUrl}`
            );
          }
          logger.verbose(`Target directory: ${targetDir}`);
        } catch (error) {
          const message = `Failed to find Scriptable directory: ${error.message}`;
          if (!continueOnError) {
            throw new Error(message);
          }
          logger.error(message);
        }
      });
      build.onEnd(async (result) => {
        if (!result.metafile || !targetDir) {
          logger.error("Missing required configuration");
          return;
        }
        try {
          const outputs = Object.keys(result.metafile.outputs).filter((file) => {
            const { entryPoint } = result.metafile.outputs[file];
            return entryPoint && filterFile(entryPoint);
          });
          for (const file of outputs) {
            const output = result.metafile.outputs[file];
            let content;
            if (result.outputFiles) {
              const outputFile = result.outputFiles.find((f) => isPathEqual(f.path, file));
              if (!outputFile) {
                throw new Error(`Output file not found: ${file}`);
              }
              content = new TextDecoder().decode(outputFile.contents);
            } else {
              await waitForFile(file);
              content = await promises.readFile(file, "utf-8");
            }
            let targetFileName = path.basename(file);
            let manifest = null;
            try {
              const inputs = Object.entries(output.inputs);
              if (!inputs.length) {
                throw new Error(`No input files found for ${file}`);
              }
              const sourceFile = inputs[0][0];
              if (!sourceFile) {
                throw new Error(`Invalid source file for ${file}`);
              }
              manifest = await tryReadManifest(sourceFile, manifestExtensions);
              if (manifest?.name) {
                targetFileName = `${manifest.name}.js`;
              }
            } catch (error) {
              logger.warn(`Failed to read manifest for ${file}: ${error}`);
            }
            const targetPath = path.join(targetDir, targetFileName);
            if (addBanner && manifest) {
              try {
                const banner = generateScriptableBanner.generateScriptableBanner(manifest);
                const newContent = banner + "\n" + content;
                await promises.writeFile(targetPath, newContent);
                logger.info(`Deployed with banner: ${file} -> ${targetPath}`);
                continue;
              } catch (error) {
                logger.warn(`Failed to add banner to ${file}: ${error}`);
              }
            }
            await promises.writeFile(targetPath, content);
            logger.info(`Deployed: ${file} -> ${targetPath}`);
          }
          logger.info(`Successfully deployed ${outputs.length} files`);
        } catch (error) {
          logger.error(`Deploy failed: ${error.message}`);
          if (!continueOnError) {
            throw error;
          }
        }
      });
    }
  };
}

exports.scriptableBanner = scriptableBanner;
exports.scriptableDeploy = scriptableDeploy;
